Imports System.Messaging

Public Class CExplore

    'purpose:   expose methods revolving around the exploring msmq

    'class-level scope
    Private c_machineName As String

    Public Sub New(ByVal machineName As String)

        'purpose:   constructor, req. a machine name to explore

        'set internal variables
        c_machineName = machineName

    End Sub

    Public Function returnQueues(ByVal isPublic As Boolean) As MessageQueue()

        'purpose: return an array of messageQueue objects for the local machine

        'determine if browsing public or private queues
        If isPublic Then

            Try

                'return public queues by machine name
                Return MessageQueue.GetPublicQueuesByMachine(machineName:=c_machineName)

            Catch

                'can't get public queues
                Err.Raise(number:=Err.Number, source:=Err.Source, _
                    description:=Err.Description)

            End Try

        Else

            Try

                'return private queues by machine name
                Return MessageQueue.GetPrivateQueuesByMachine(machineName:=c_machineName)

            Catch

                'can't get private queues
                Err.Raise(number:=Err.Number, source:=Err.Source, _
                    description:=Err.Description)

            End Try

        End If

    End Function

    Public Function returnQueue(ByVal qPath As String) As MessageQueue

        'purpose: return queue object based on guid

        Try

            'return the requested queue
            Return New MessageQueue(path:=c_machineName & "\" & qPath)

        Catch

            'can't get queue, raise error
            Err.Raise(number:=Err.Number, source:=Err.Source, _
                description:=Err.Description)

        End Try

    End Function

    Public Function returnMessages(ByVal qPath As String) As Message()

        'purpose: return a snapshot of all the messages in a given queue

        'local scope
        Dim queue As MessageQueue

        Try

            'get a reference to the queue
            queue = New MessageQueue(path:=c_machineName & "\" & qPath)

            'set the property read filter to return all properties
            queue.MessageReadPropertyFilter.SetAll()

            'tell the queue the message format to return
            CType(queue.Formatter, XmlMessageFormatter).TargetTypeNames = _
                New String() {"System.String"}

            'return the messages
            Return queue.GetAllMessages()

        Catch

            'raise error to calling client
            Err.Raise(number:=Err.Number, source:=Err.Source, _
                description:=Err.Description)

        End Try

    End Function

    Public Function returnMessage(ByVal qPath As String, _
        ByVal msgId As String) As Message

        'purpose: return a snapshot of a given message

        'local scope
        Dim queue As MessageQueue

        Try

            'get a reference to the queue
            queue = New MessageQueue(path:=c_machineName & "\" & qPath)

            'tell the queue the message format to return
            CType(queue.Formatter, XmlMessageFormatter).TargetTypeNames = _
                New String() {"System.String"}

            'set the property read filter to return all properties
            queue.MessageReadPropertyFilter.SetAll()

            'return the messages
            Return queue.PeekById(id:=msgId)

        Catch

            'raise error to calling client
            Err.Raise(number:=Err.Number, source:=Err.Source, _
                description:=Err.Description)

        End Try

    End Function

End Class
