Imports System.Messaging
Imports System.Diagnostics

'purpose:   This class is used to receive and reject messages from here, you might
'           enforce business rules or call other classes that enforce buisness 
'           rules.

'           Additionally, you might log information to the event log, send special 
'           messages to other queues, etc.

'           Right now, the code simply receives messages and writes them to the 
'           Event log()

Public Class CTrx

    'class-level scope
    Private queue As MessageQueue

    Public Sub New(ByVal machineName As String, ByVal qPath As String)

        'purpose:   constructor, req. a machine name and queue

        'get a class-level reference to a the queue
        Try
            queue = New MessageQueue(path:=machineName & "\" & qPath)
        Catch
            Err.Raise(number:=Err.Number, description:=Err.Description)
        End Try

        'set the property read filter to return all properties
        queue.MessageReadPropertyFilter.SetAll()

        'set the formatter for the queue to use
        CType(queue.Formatter, XmlMessageFormatter).TargetTypeNames = _
            New String() {"System.String"}

    End Sub

    Public Sub receiveTrxMsg(ByVal trxId As String)

        'purpose: receive messages as a transaction

        'local scope
        Dim msg() As Message
        Dim trx As New MessageQueueTransaction()
        Dim i As Integer
        Dim msgRec As Message

        'get a snapshot of messages in the queue as an array
        'note: the array is in the same order as msgs in the queue
        '      therefore, trx can be assumed sequential as they were written
        msg = queue.GetAllMessages

        'start a transaction
        trx.Begin()

        'loop through our snapshot of messages and pull out the transaction
        For i = 0 To UBound(msg)

            'check the transaction id property
            If msg(i).TransactionId = trxId Then

                Try

                    'receive the message as part of a transaction
                    msgRec = queue.ReceiveById(id:=msg(i).Id, transaction:=trx)

                Catch

                    'error, abort the transaction
                    trx.Abort()
                    Err.Raise(number:=Err.Number, description:=Err.Description)

                End Try

                'do something with the message, business rules, write to db, etc.
                'write the item to the event log
                EventLog.WriteEntry(source:="messageManager", _
                    Message:="Message Received: " & msgRec.Body, _
                    type:=EventLogEntryType.Information)

                'short-circuit the loop if end of trx
                If msg(i).IsLastInTransaction Then Exit For

            End If

        Next

        'commit the transaction
        trx.Commit()

    End Sub

    Public Sub rejectTrxMsg(ByVal trxId As String)

        'purpose: reject a group of messages as trx

        'local scope
        Dim msg() As Message
        Dim trx As New MessageQueueTransaction()
        Dim i As Integer
        Dim msgRec As Message

        'get a snapshot of messages in the queue as an array
        'note: the array is in the same order as msgs in the queue
        '      therefore, trx can be assumed sequential as they were written
        msg = queue.GetAllMessages

        'start a transaction
        trx.Begin()

        'loop through our snapshot of messages and pull out the transaction
        For i = 0 To UBound(msg)

            'check the transaction id property
            If msg(i).TransactionId = trxId Then

                Try

                    'receive the message as part of a transaction
                    msgRec = queue.ReceiveById(id:=msg(i).Id, transaction:=trx)

                Catch

                    'error, abort the transaction
                    trx.Abort()
                    Err.Raise(number:=Err.Number, description:=Err.Description)

                End Try

                'do something with the message, business rules, write to db, etc.
                'write the item to the event log
                EventLog.WriteEntry(source:="messageManager", _
                    Message:="Message Rejected: " & msgRec.Body, _
                    type:=EventLogEntryType.Information)

                'short-circuit the loop if end of trx
                If msg(i).IsLastInTransaction Then Exit For

            End If

        Next

        'commit the transaction
        trx.Commit()

    End Sub

    Public Sub receiveMsg(ByVal msgId As String)

        'purpose: receive single message 

        'local scope 
        Dim msg As Message
        Dim eventLog As New EventLog()

        'get the message
        msg = queue.ReceiveById(id:=msgId)

        'do something with the message, business rules, write to db, etc.
        'write the item to the event log
        eventLog.WriteEntry(source:="messageManager", _
            Message:="Message Received: " & msg.Body, _
            type:=EventLogEntryType.Information)

    End Sub

    Public Sub rejectMsg(ByVal msgId As String)

        'purpose: reject a single message

        'local scope 
        Dim msg As Message
        Dim eventLog As New EventLog()

        'get the message
        msg = queue.ReceiveById(id:=msgId)

        'do something with the message, business rules, write to db, etc.

        'write the item to the event log
        eventLog.WriteEntry(source:="messageManager", _
            Message:="Message Rejected: " & msg.Body, _
            type:=EventLogEntryType.Information)

    End Sub

End Class
