Imports System.Messaging

Public Class receiveExample

  Public Shared Function peekAndWait( _
      ByVal queuePath As String, _
      ByVal msgLabel As String) As Message

    'purpose:   peek at messages as the arrive in the queue
    '           if they contain the label (msgLabel) return the 
    '           message, else remove the message from the queue 
    '           and continue to wait

    'local scope
    Dim queue As New MessageQueue(path:=queuePath)
    Dim msg As Message

    'set the message properties that we are intrested in reading
    With queue.MessageReadPropertyFilter

      'clear the filter (sets all to false)
      .ClearAll()

      'indicate that we want to read the label property
      .Label = True

    End With

    'call the peak method
    'note: processing is blocked until a message arrives in the queue
    msg = queue.Peek

    'we have a message, read its label
    If msg.Label <> msgLabel Then

      'write the label to the console
      Console.WriteLine(msg.Label)

      'pull the message from the server
      queue.Receive()

      'use recursion to wait for a valid message
      msg = peekAndWait(queuePath:=queuePath, msgLabel:=msgLabel)

    Else

      'return a valid message
      queue.MessageReadPropertyFilter.SetAll()
      msg = queue.Receive

    End If

    'function return
    Return msg

  End Function

End Class

Public Class sendExample

  Public Shared Sub sendMessage( _
      ByVal queuePath As String, _
      ByVal msg As Object, _
      ByVal msgLabel As String)

    'purpose:   send a message to a queue

    'local scope
    Dim queue As New MessageQueue()

    'check to see if the queue exists
    If queue.Exists(path:=queuePath) Then

      'reference the physical queue
      queue.Path = queuePath

      'send the message to the queue
      queue.Send(obj:=msg, label:=msgLabel)

    Else
      'raise some exception
    End If

  End Sub

End Class

Module Module1

  Sub main()

    'purpose: call the function(s) & write the result to the console

    'purge the queue
    Dim queue As MessageQueue
    queue = New MessageQueue(path:=".\Private$\myqueue")
    queue.Purge()
    queue.Close()

    'add a few messages to the queue
    sendExample.sendMessage( _
        queuePath:=".\Private$\myqueue", msg:="Test", _
        msgLabel:="NOT ORDER 1")

    sendExample.sendMessage( _
        queuePath:=".\Private$\myqueue", msg:="Test", _
        msgLabel:="NOT ORDER 2")

    sendExample.sendMessage( _
        queuePath:=".\Private$\myqueue", msg:="Test", _
        msgLabel:="ORDER")

    sendExample.sendMessage( _
        queuePath:=".\Private$\myqueue", msg:="Test", _
        msgLabel:="NOT ORDER 3")

    'peek at messages and return the correct one
    Console.WriteLine( _
        receiveExample.peekAndWait( _
        queuePath:=".\Private$\myqueue", msgLabel:="ORDER").Body)

    'wait for the user to stop the console application
    'this allows you to view the results
    Console.WriteLine("Enter 's' to stop the application.")

    'loop until users presses s key
    Do While Console.ReadLine <> "s" : Loop

  End Sub

End Module
